@@ -43,16 +43,23 @@
 #include "qqmlcontext.h"
 #include "qqmlinfo.h"
 #include "qqmldata_p.h"
+
+#include <private/qqmldebugserviceinterfaces_p.h>
+#include <private/qqmldebugconnector_p.h>
+
 #include <private/qqmlprofiler_p.h>
 #include <private/qqmlexpression_p.h>
 #include <private/qqmlscriptstring_p.h>
 #include <private/qqmlbuiltinfunctions_p.h>
 #include <private/qqmlvmemetaobject_p.h>
 #include <private/qqmlvaluetypewrapper_p.h>
+#include <private/qv4qmlcontext_p.h>
 #include <private/qv4qobjectwrapper_p.h>
 #include <private/qv4variantobject_p.h>
 #include <private/qv4jscall_p.h>
 
+#include <qtqml_tracepoints_p.h>
+
 #include <QVariant>
 #include <QtCore/qdebug.h>
 #include <QVector>
@@ -181,6 +188,8 @@ void QQmlBinding::update(QQmlPropertyData::WriteFlags flags)
     if (canUseAccessor())
         flags.setFlag(QQmlPropertyData::BypassInterceptor);
 
+    Q_TRACE_SCOPE(QQmlBinding, engine, function() ? function()->name()->toQString() : QString(),
+                  sourceLocation().sourceFile, sourceLocation().line, sourceLocation().column);
     QQmlBindingProfiler prof(QQmlEnginePrivate::get(engine)->profiler, function());
     doUpdate(watcher, flags, scope);
 
@@ -297,8 +306,9 @@ protected:
             case QMetaType::Int:
                 if (result.isInteger())
                     return doStore<int>(result.integerValue(), pd, flags);
-                else if (result.isNumber())
-                    return doStore<int>(result.doubleValue(), pd, flags);
+                else if (result.isNumber()) {
+                    return doStore<int>(QV4::StaticValue::toInteger(result.doubleValue()), pd, flags);
+                }
                 break;
             case QMetaType::Double:
                 if (result.isNumber())
@@ -314,7 +324,7 @@ protected:
                 break;
             default:
                 if (const QV4::QQmlValueTypeWrapper *vtw = result.as<const QV4::QQmlValueTypeWrapper>()) {
-                    if (vtw->d()->valueType->typeId == pd->propType()) {
+                    if (vtw->d()->valueType()->metaType.id() == pd->propType()) {
                         return vtw->write(m_target.data(), pd->coreIndex());
                     }
                 }
@@ -335,7 +345,7 @@ protected:
 
 class QQmlTranslationBinding : public GenericBinding<QMetaType::QString> {
 public:
-    QQmlTranslationBinding(const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &compilationUnit, const QV4::CompiledData::Binding *binding)
+    QQmlTranslationBinding(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit, const QV4::CompiledData::Binding *binding)
     {
         setCompilationUnit(compilationUnit);
         m_binding = binding;
@@ -356,7 +366,7 @@ public:
         if (!isAddedToObject() || hasError())
             return;
 
-        const QString result = m_binding->valueAsString(m_compilationUnit.data());
+        const QString result = m_compilationUnit->bindingValueAsString(m_binding);
 
         Q_ASSERT(targetObject());
 
@@ -378,7 +388,7 @@ private:
     const QV4::CompiledData::Binding *m_binding;
 };
 
-QQmlBinding *QQmlBinding::createTranslationBinding(const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &unit, const QV4::CompiledData::Binding *binding, QObject *obj, QQmlContextData *ctxt)
+QQmlBinding *QQmlBinding::createTranslationBinding(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &unit, const QV4::CompiledData::Binding *binding, QObject *obj, QQmlContextData *ctxt)
 {
     QQmlTranslationBinding *b = new QQmlTranslationBinding(unit, binding);
 
@@ -386,6 +396,11 @@ QQmlBinding *QQmlBinding::createTranslationBinding(const QQmlRefPointer<QV4::Com
     b->QQmlJavaScriptExpression::setContext(ctxt);
     b->setScopeObject(obj);
 
+    if (QQmlDebugTranslationService *service
+                 = QQmlDebugConnector::service<QQmlDebugTranslationService>()) {
+        service->foundTranslationBinding(b, obj, ctxt);
+    }
+
     return b;
 }
 
@@ -473,7 +488,7 @@ Q_NEVER_INLINE bool QQmlBinding::slowWrite(const QQmlPropertyData &core,
                 if (!propertyMetaObject.isNull())
                     propertyType = propertyMetaObject.className();
             }
-        } else if (userType != QVariant::Invalid) {
+        } else if (userType != QMetaType::UnknownType) {
             if (userType == QMetaType::Nullptr || userType == QMetaType::VoidStar)
                 valueType = "null";
             else
@@ -517,9 +532,9 @@ QString QQmlBinding::expressionIdentifier() const
 {
     if (auto f = function()) {
         QString url = f->sourceFile();
-        quint16 lineNumber = f->compiledFunction->location.line;
-        quint16 columnNumber = f->compiledFunction->location.column;
-        return url + QString::asprintf(":%u:%u", uint(lineNumber), uint(columnNumber));
+        uint lineNumber = f->compiledFunction->location.line;
+        uint columnNumber = f->compiledFunction->location.column;
+        return url + QString::asprintf(":%u:%u", lineNumber, columnNumber);
     }
 
     return QStringLiteral("[native code]");
@@ -630,7 +645,10 @@ void QQmlBinding::getPropertyData(QQmlPropertyData **propertyData, QQmlPropertyD
         Q_ASSERT(valueTypeMetaObject);
         QMetaProperty vtProp = valueTypeMetaObject->property(m_targetIndex.valueTypeIndex());
         valueTypeData->setFlags(QQmlPropertyData::flagsForProperty(vtProp));
+
+        // valueTypeData is expected to be local here. It must not be shared with other threads.
         valueTypeData->setPropType(vtProp.userType());
+
         valueTypeData->setCoreIndex(m_targetIndex.valueTypeIndex());
     }
 }
@@ -733,7 +751,11 @@ QQmlBinding *QQmlBinding::newBinding(QQmlEnginePrivate *engine, const QQmlProper
     if (property && property->isQObject())
         return new QObjectPointerBinding(engine, property->propType());
 
-    const int type = (property && property->isFullyResolved()) ? property->propType() : QMetaType::UnknownType;
+    // If the property is not resolved at this point, you get a binding of unknown type.
+    // This has been the case for a long time and we keep it like this in Qt5 to be bug-compatible.
+    const int type = (property && property->isResolved())
+            ? property->propType()
+            : QMetaType::UnknownType;
 
     if (type == qMetaTypeId<QQmlBinding *>()) {
         return new QQmlBindingBinding;
