@@ -598,12 +598,8 @@ QObject *QQmlProperty::object() const
 */
 QQmlProperty &QQmlProperty::operator=(const QQmlProperty &other)
 {
-    if (d)
-        d->release();
-    d = other.d;
-    if (d)
-        d->addref();
-
+    QQmlProperty copied(other);
+    qSwap(d, copied.d);
     return *this;
 }
 
@@ -918,6 +914,8 @@ QQmlPropertyPrivate::signalExpression(const QQmlProperty &that)
     if (!(that.type() & QQmlProperty::SignalProperty))
         return nullptr;
 
+    if (!that.d->object)
+        return nullptr;
     QQmlData *data = QQmlData::get(that.d->object);
     if (!data)
         return nullptr;
@@ -957,6 +955,8 @@ void QQmlPropertyPrivate::takeSignalExpression(const QQmlProperty &that,
         return;
     }
 
+    if (!that.d->object)
+        return;
     QQmlData *data = QQmlData::get(that.d->object, nullptr != expr);
     if (!data)
         return;
