@@ -37,17 +37,21 @@
 **
 ****************************************************************************/
 
-#include "quicktest.h"
+#include "quicktest_p.h"
 #include "quicktestresult_p.h"
 #include <QtTest/qtestsystem.h>
 #include "qtestoptions_p.h"
 #include <QtQml/qqml.h>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcontext.h>
+#include <QtQuick/private/qquickitem_p.h>
+#include <QtQuick/qquickitem.h>
 #include <QtQuick/qquickview.h>
 #include <QtQml/qjsvalue.h>
 #include <QtQml/qjsengine.h>
 #include <QtQml/qqmlpropertymap.h>
+#include <QtQuick/private/qquickitem_p.h>
+#include <QtQuick/qquickitem.h>
 #include <QtGui/qopengl.h>
 #include <QtCore/qurl.h>
 #include <QtCore/qfileinfo.h>
@@ -66,6 +70,7 @@
 #include <QtQml/QQmlFileSelector>
 
 #include <private/qqmlcomponent_p.h>
+#include <private/qv4executablecompilationunit_p.h>
 
 #ifdef QT_QMLTEST_WITH_WIDGETS
 #include <QtWidgets/QApplication>
@@ -73,52 +78,60 @@
 
 QT_BEGIN_NAMESPACE
 
-class QTestRootObject : public QObject
+/*!
+    \since 5.13
+
+    Returns \c true if \l {QQuickItem::}{updatePolish()} has not been called
+    on \a item since the last call to \l {QQuickItem::}{polish()},
+    otherwise returns \c false.
+
+    When assigning values to properties in QML, any layouting the item
+    must do as a result of the assignment might not take effect immediately,
+    but can instead be postponed until the item is polished. For these cases,
+    you can use this function to ensure that the item has been polished
+    before the execution of the test continues. For example:
+
+    \code
+        QVERIFY(QQuickTest::qIsPolishScheduled(item));
+        QVERIFY(QQuickTest::qWaitForItemPolished(item));
+    \endcode
+
+    Without the call to \c qIsPolishScheduled() above, the
+    call to \c qWaitForItemPolished() might see that no polish
+    was scheduled and therefore pass instantly, assuming that
+    the item had already been polished. This function
+    makes it obvious why an item wasn't polished and allows tests to
+    fail early under such circumstances.
+
+    The QML equivalent of this function is
+    \l {TestCase::}{isPolishScheduled()}.
+
+    \sa QQuickItem::polish(), QQuickItem::updatePolish()
+*/
+bool QQuickTest::qIsPolishScheduled(const QQuickItem *item)
 {
-    Q_OBJECT
-    Q_PROPERTY(bool windowShown READ windowShown NOTIFY windowShownChanged)
-    Q_PROPERTY(bool hasTestCase READ hasTestCase WRITE setHasTestCase NOTIFY hasTestCaseChanged)
-    Q_PROPERTY(QObject *defined READ defined)
-public:
-    QTestRootObject(QObject *parent = nullptr)
-        : QObject(parent), hasQuit(false), m_windowShown(false), m_hasTestCase(false)  {
-        m_defined = new QQmlPropertyMap(this);
-#if defined(QT_OPENGL_ES_2_ANGLE)
-        m_defined->insert(QLatin1String("QT_OPENGL_ES_2_ANGLE"), QVariant(true));
-#endif
-    }
+    return QQuickItemPrivate::get(item)->polishScheduled;
+}
 
-    static QTestRootObject *instance() {
-        static QPointer<QTestRootObject> object = new QTestRootObject;
-        if (!object) {
-            // QTestRootObject was deleted when previous test ended, create a new one
-            object = new QTestRootObject;
-        }
-        return object;
-    }
+/*!
+    \since 5.13
 
-    bool hasQuit:1;
-    bool hasTestCase() const { return m_hasTestCase; }
-    void setHasTestCase(bool value) { m_hasTestCase = value; emit hasTestCaseChanged(); }
+    Waits for \a timeout milliseconds or until
+    \l {QQuickItem::}{updatePolish()} has been called on \a item.
 
-    bool windowShown() const { return m_windowShown; }
-    void setWindowShown(bool value) { m_windowShown = value; emit windowShownChanged(); }
-    QQmlPropertyMap *defined() const { return m_defined; }
+    Returns \c true if \c updatePolish() was called on \a item within
+    \a timeout milliseconds, otherwise returns \c false.
 
-    void init() { setWindowShown(false); setHasTestCase(false); hasQuit = false; }
+    The QML equivalent of this function is
+    \l {TestCase::}{waitForItemPolished()}.
 
-Q_SIGNALS:
-    void windowShownChanged();
-    void hasTestCaseChanged();
-
-private Q_SLOTS:
-    void quit() { hasQuit = true; }
-
-private:
-    bool m_windowShown : 1;
-    bool m_hasTestCase :1;
-    QQmlPropertyMap *m_defined;
-};
+    \sa QQuickItem::polish(), QQuickItem::updatePolish(),
+        QQuickTest::qIsPolishScheduled()
+*/
+bool QQuickTest::qWaitForItemPolished(const QQuickItem *item, int timeout)
+{
+    return QTest::qWaitFor([&]() { return !QQuickItemPrivate::get(item)->polishScheduled; }, timeout);
+}
 
 static QObject *testRootObject(QQmlEngine *engine, QJSEngine *jsEngine)
 {
@@ -231,7 +244,8 @@ public:
         m_errors += component.errors();
 
         if (component.isReady()) {
-            QQmlRefPointer<CompilationUnit> rootCompilationUnit = QQmlComponentPrivate::get(&component)->compilationUnit;
+            QQmlRefPointer<QV4::ExecutableCompilationUnit> rootCompilationUnit
+                    = QQmlComponentPrivate::get(&component)->compilationUnit;
             TestCaseEnumerationResult result = enumerateTestCases(rootCompilationUnit.data());
             m_testCases = result.testCases + result.finalizedPartialTestCases();
             m_errors += result.errors;
@@ -271,12 +285,14 @@ private:
         }
     };
 
-    TestCaseEnumerationResult enumerateTestCases(CompilationUnit *compilationUnit, const Object *object = nullptr)
+    TestCaseEnumerationResult enumerateTestCases(
+            const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
+            const Object *object = nullptr)
     {
         QQmlType testCaseType;
         for (quint32 i = 0, count = compilationUnit->importCount(); i < count; ++i) {
             const Import *import = compilationUnit->importAt(i);
-            if (compilationUnit->stringAt(import->uriIndex) != QLatin1Literal("QtTest"))
+            if (compilationUnit->stringAt(import->uriIndex) != QLatin1String("QtTest"))
                 continue;
 
             QString testCaseTypeName(QStringLiteral("TestCase"));
@@ -293,18 +309,22 @@ private:
 
         if (!object) // Start at root of compilation unit if not enumerating a specific child
             object = compilationUnit->objectAt(0);
+        if (object->flags & Object::IsInlineComponentRoot)
+            return result;
 
-        if (CompilationUnit *superTypeUnit = compilationUnit->resolvedTypes.value(object->inheritedTypeNameIndex)->compilationUnit.data()) {
+        if (const auto superTypeUnit = compilationUnit->resolvedTypes.value(
+                    object->inheritedTypeNameIndex)->compilationUnit()) {
             // We have a non-C++ super type, which could indicate we're a subtype of a TestCase
             if (testCaseType.isValid() && superTypeUnit->url() == testCaseType.sourceUrl())
                 result.isTestCase = true;
-            else
+            else if (superTypeUnit->url() != compilationUnit->url()) { // urls are the same for inline component, avoid infinite recursion
                 result = enumerateTestCases(superTypeUnit);
+            }
 
             if (result.isTestCase) {
                 // Look for override of name in this type
                 for (auto binding = object->bindingsBegin(); binding != object->bindingsEnd(); ++binding) {
-                    if (compilationUnit->stringAt(binding->propertyNameIndex) == QLatin1Literal("name")) {
+                    if (compilationUnit->stringAt(binding->propertyNameIndex) == QLatin1String("name")) {
                         if (binding->type == QV4::CompiledData::Binding::Type_String) {
                             result.testCaseName = compilationUnit->stringAt(binding->stringIndex);
                         } else {
@@ -323,10 +343,10 @@ private:
                 auto functionsEnd = compilationUnit->objectFunctionsEnd(object);
                 for (auto function = compilationUnit->objectFunctionsBegin(object); function != functionsEnd; ++function) {
                     QString functionName = compilationUnit->stringAt(function->nameIndex);
-                    if (!(functionName.startsWith(QLatin1Literal("test_")) || functionName.startsWith(QLatin1Literal("benchmark_"))))
+                    if (!(functionName.startsWith(QLatin1String("test_")) || functionName.startsWith(QLatin1String("benchmark_"))))
                         continue;
 
-                    if (functionName.endsWith(QLatin1Literal("_data")))
+                    if (functionName.endsWith(QLatin1String("_data")))
                         continue;
 
                     result.testFunctions << functionName;
@@ -440,7 +460,7 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
     }
 #endif
 
-#if defined(Q_OS_ANDROID) || defined(Q_OS_WINRT)
+#if defined(Q_OS_WINRT)
     if (testPath.isEmpty())
         testPath = QLatin1String(":/");
 #endif
@@ -451,6 +471,12 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
         if (QFile::exists(s))
             testPath = s;
     }
+
+#if defined(Q_OS_ANDROID)
+            if (testPath.isEmpty())
+                    testPath = QLatin1String(":/");
+#endif
+
     if (testPath.isEmpty()) {
         QDir current = QDir::current();
 #ifdef Q_OS_WIN
@@ -492,10 +518,10 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
 
     qputenv("QT_QTESTLIB_RUNNING", "1");
 
-    // Register the test object
+    // Register the custom factory function
     qmlRegisterSingletonType<QTestRootObject>("Qt.test.qtestroot", 1, 0, "QTestRootObject", testRootObject);
 
-    QSet<QString> commandLineTestFunctions = QTest::testFunctions.toSet();
+    QSet<QString> commandLineTestFunctions(QTest::testFunctions.cbegin(), QTest::testFunctions.cend());
     const bool filteringTestFunctions = !commandLineTestFunctions.isEmpty();
 
     // Scan through all of the "tst_*.qml" files and run each of them
@@ -537,7 +563,7 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
             continue;
         }
 
-        const QSet<QString> availableTestSet = availableTestFunctions.toSet();
+        const QSet<QString> availableTestSet(availableTestFunctions.cbegin(), availableTestFunctions.cend());
         if (filteringTestFunctions && !availableTestSet.intersects(commandLineTestFunctions))
             continue;
         commandLineTestFunctions.subtract(availableTestSet);
@@ -624,5 +650,3 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
 }
 
 QT_END_NAMESPACE
-
-#include "quicktest.moc"
