@@ -2,15 +2,15 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.3-4-116
+es5id: 15.2.3.3-4-120
 description: >
     Object.getOwnPropertyDescriptor returns data desc for functions on
-    built-ins (Date.prototype.constructor)
+    built-ins (Date.prototype.getFullYear)
 ---*/
 
-var desc = Object.getOwnPropertyDescriptor(Date.prototype, "constructor");
+var desc = Object.getOwnPropertyDescriptor(Date.prototype, "getFullYear");
 
-assert.sameValue(desc.value, Date.prototype.constructor, 'desc.value');
+assert.sameValue(desc.value, Date.prototype.getFullYear, 'desc.value');
 assert.sameValue(desc.writable, true, 'desc.writable');
 assert.sameValue(desc.enumerable, false, 'desc.enumerable');
 assert.sameValue(desc.configurable, true, 'desc.configurable');
