@@ -2,10 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-11
+es5id: 15.2.3.2-2-27
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Error)
+    (Error object)
 ---*/
 
-assert.sameValue(Object.getPrototypeOf(Error), Function.prototype, 'Object.getPrototypeOf(Error)');
+var obj = new Error();
+
+assert.sameValue(Object.getPrototypeOf(obj), Error.prototype, 'Object.getPrototypeOf(obj)');
